#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Created by pat on 3/9/18
"""
.. currentmodule:: luijo.errors
.. moduleauthor:: Pat Daburu <pat@daburu.net>

Something went wrong?  It did.
"""

from abc import ABCMeta


class LuijoException(Exception):
    """
    This is a common base class for all custom luijo exceptions.
    """
    __metaclass__ = ABCMeta

    def __init__(self, message: str, inner: Exception = None):
        """

        :param message: the exception message
        :param inner: the exception that caused this exception
        """
        super().__init__(message)
        self._message: str = message
        self._inner: Exception = inner

    @property
    def message(self) -> str:
        """
        Get the exception message.
        :return: the exception message
        """
        return self._message

    @property
    def inner(self) -> Exception:
        """
        Get the inner exception that caused this exception.
        :return: the inner exception
        """
        return self._inner



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Created by pat on 4/21/18
"""
.. currentmodule:: luijo.config
.. moduleauthor:: Pat Daburu <pat@daburu.net>

Say something descriptive about the 'config' module.
"""
import logging
import os
from typing import cast
from typing import Iterable
import luigi


def find_configs() -> Iterable[str]:
    """
    This function attempts to find the current Luigi
    :return:
    """
    # Create a list to hold the config paths as we find them.
    configs = []
    if 'LUIGI_CONFIG_PATH' in os.environ:
        configs.append(os.environ['LUIGI_CONFIG_PATH'])
    else:
        for candidate in [
                os.path.join(os.getcwd(), 'luiji.cfg'),
                os.path.join(os.getcwd(), 'client.cfg'),
                '/etc/luigi/client.cfg'
        ]:
            if os.path.exists(candidate) and os.path.isfile(candidate):
                configs.append(candidate)
    return configs


class FileSystem(luigi.Config):
    """
    These are configuration settings to which we can refer when working with
    the local file system.

    :cvar target_home_dir: the default home directory for local targets
    """
    target_home_dir: luigi.Parameter = luigi.Parameter(default=os.getcwd())


# Attempt to create the local home directory for targets.
try:
    os.makedirs(cast(str, FileSystem().target_home_dir), exist_ok=True)
except OSError:
    logging.exception(f'Target home directory creation failed '
                      f'({FileSystem().target_home_dir})')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. currentmodule:: luijo.tasks
.. moduleauthor:: Pat Daburu <pat@daburu.net>

Luigi targets, plus just a little more.
"""

from abc import ABCMeta
from enum import Enum
import pickle
import os
import jsonpickle
import luigi.target


class Serialization(Enum):
    """
    These are the supported serialization methods.
    """
    JSON = 'json'
    BINARY = 'binary'


def touch(local_target: luigi.LocalTarget):
    """
    You can use this function to create a local target file in cases where you
    just need it to signify that the task completed.

    :param local_target: the local target
    """
    with local_target.open('w') as target:
        target.write('')


class LocalObjectTarget(luigi.LocalTarget):
    """
    This is a local target you can use to serialize a Python object to a file.
    """
    __metaclass__ = ABCMeta

    def deserialize(self):
        """
        Retrieve the target object.

        :return: the target object
        """
        # We'll try to use jsonpickle first...
        try:
            with self.open('r') as fin:
                frozen = fin.read()
                thawed = jsonpickle.decode(frozen)
                return thawed
        except UnicodeDecodeError:
            # If the attempt to unpickle using JSON fails, we assume we have a
            # binary file.
            with open(self.path, 'rb') as fin:
                unpickled = pickle.load(fin)
                return unpickled

    def serialize(self,
                  obj,
                  format_: Serialization = Serialization.BINARY):
        """
        Serialize an object to the local target.

        :param obj: the object you want to serialize
        :param format_: the serialization format
        """
        if format_ == Serialization.JSON:
            # Encode the object.
            frozen = jsonpickle.encode(obj)
            # Open the output file for writing.
            with self.open('w') as fout:
                # Write the encoded object.
                fout.write(frozen)
        elif format_ == Serialization.BINARY:
            # Pickle the object to the file.
            os.makedirs(
                os.path.dirname(
                    os.path.abspath(
                        os.path.expanduser(
                            self.path
                        )
                    )
                ), exist_ok=True)
            with open(self.path, 'wb') as fout:
                pickle.dump(obj, fout, pickle.HIGHEST_PROTOCOL)
        else:  # pragma: no cover
            raise NotImplementedError(
                f'Unsupported serialization: { format_.name }')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Created by pat on 4/21/18
"""
.. currentmodule:: luijo.mock
.. moduleauthor:: Pat Daburu <pat@daburu.net>

Say something descriptive about the 'testing' module.
"""
import luigi
import luigi.mock


class MockTask(luigi.Task):
    """
    This is a mock task.  It has no requirements, performs no actual work, and
    specifies a :py:class:`luigi.mock.MockTarget` as its output.  Use it when
    it might be handy.
    """
    def requires(self):
        """
        This task has no requirements.

        :return: an empty iteration
        """
        return []

    def output(self) -> luigi.mock.MockTarget:
        """
        This implementation returns a mock target.

        :return: a mock target
        """
        return luigi.mock.MockTarget(None)

    def run(self):
        """
        Running the task has no effect.
        """
        pass
